# coding: utf-8

"""
    Mentortools Public API

    Mentortools public API service

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from public_api_server_connector_v2.models.course_submodule import CourseSubmodule
from public_api_server_connector_v2.models.module_lesson import ModuleLesson
from typing import Optional, Set
from typing_extensions import Self
from pydantic_core import to_jsonable_python

class CourseModule(BaseModel):
    """
    CourseModule
    """ # noqa: E501
    mandatory: Optional[StrictBool] = False
    is_published: Optional[StrictBool] = False
    is_active: Optional[StrictBool] = False
    title: StrictStr
    public_description: Optional[StrictStr] = None
    short_description: Optional[StrictStr] = None
    image_id: Optional[StrictStr] = None
    available_at: Optional[StrictInt] = Field(default=None, description="Timestamp in unix milliseconds when the module becomes available")
    id: StrictInt
    course_id: StrictInt
    order: StrictInt
    submodules: List[CourseSubmodule]
    lessons: List[ModuleLesson]
    __properties: ClassVar[List[str]] = ["mandatory", "is_published", "is_active", "title", "public_description", "short_description", "image_id", "available_at", "id", "course_id", "order", "submodules", "lessons"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_by_alias=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(to_jsonable_python(self.to_dict()))

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CourseModule from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in submodules (list)
        _items = []
        if self.submodules:
            for _item_submodules in self.submodules:
                if _item_submodules:
                    _items.append(_item_submodules.to_dict())
            _dict['submodules'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in lessons (list)
        _items = []
        if self.lessons:
            for _item_lessons in self.lessons:
                if _item_lessons:
                    _items.append(_item_lessons.to_dict())
            _dict['lessons'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CourseModule from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "mandatory": obj.get("mandatory") if obj.get("mandatory") is not None else False,
            "is_published": obj.get("is_published") if obj.get("is_published") is not None else False,
            "is_active": obj.get("is_active") if obj.get("is_active") is not None else False,
            "title": obj.get("title"),
            "public_description": obj.get("public_description"),
            "short_description": obj.get("short_description"),
            "image_id": obj.get("image_id"),
            "available_at": obj.get("available_at"),
            "id": obj.get("id"),
            "course_id": obj.get("course_id"),
            "order": obj.get("order"),
            "submodules": [CourseSubmodule.from_dict(_item) for _item in obj["submodules"]] if obj.get("submodules") is not None else None,
            "lessons": [ModuleLesson.from_dict(_item) for _item in obj["lessons"]] if obj.get("lessons") is not None else None
        })
        return _obj


